from rest_framework import serializers


def get_first_matching_attr(obj, *attrs, default=None):
    for attr in attrs:
        if hasattr(obj, attr):
            return getattr(obj, attr)

    return default


def create_serializer_class(name, fields):
    return type(name, (serializers.Serializer, ), fields)


def inline_serializer(*, fields, data=None, **kwargs):
    serializer_class = create_serializer_class(name='', fields=fields)

    if data is not None:
        return serializer_class(data=data, **kwargs)

    return serializer_class(**kwargs)



import uuid
from decimal import Decimal

from django.core.validators import MinValueValidator
from django.db import models
from ool import VersionedMixin, VersionField


class TicketBase(VersionedMixin, models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField("Title", max_length=50)
    price = models.DecimalField(
        "Price",
        max_digits=10,
        decimal_places=4,
        validators=[MinValueValidator(Decimal("0.01"))],
    )
    version = VersionField()

    class Meta:
        abstract = True


class OrderBase(VersionedMixin, models.Model):
    CREATED = "created"
    CANCELLED = "cancelled"
    AWAITINGPAYMENT = "awaiting_payment"
    COMPLETE = "complete"
    STATUS_CHOICES = [
        (CREATED, "created"),
        (CANCELLED, "cancelled"),
        (AWAITINGPAYMENT, "awaiting_payment"),
        (COMPLETE, "complete"),
    ]

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user_id = models.CharField("User ID", max_length=36)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default=CREATED)
    version = VersionField()

    class Meta:
        abstract = True



import json

import boto3
from django.conf import settings


def sqs_publish(topic_name, body):
    sns_client_options = {
        "region_name": settings.AWS_REGION_NAME,
        "aws_access_key_id": settings.AWS_ACCESS_KEY_ID,
        "aws_secret_access_key": settings.AWS_SECRET_ACCESS_KEY,
        "endpoint_url": settings.AWS_SNS_ENDPOINT_URL,
    }
    sns_client = boto3.client("sns", **sns_client_options)
    sns_client.publish(
        TopicArn=settings.AWS_TOPIC_ARN_BASE + topic_name,
        Subject=topic_name,
        Message=json.dumps(body),
    )



import json
from signal import SIGINT, SIGTERM, signal

import boto3
from django.conf import settings


class SignalHandler:
    def __init__(self):
        self.received_signal = False
        signal(SIGINT, self._signal_handler)
        signal(SIGTERM, self._signal_handler)

    def _signal_handler(self, signal, frame):
        print(f"handling signal {signal}, exiting gracefully")
        self.received_signal = True


def sqs_consume(exchanges):
    sqs_client_options = {
        "region_name": settings.AWS_REGION_NAME,
        "aws_access_key_id": settings.AWS_ACCESS_KEY_ID,
        "aws_secret_access_key": settings.AWS_SECRET_ACCESS_KEY,
        "endpoint_url": settings.AWS_SQS_ENDPOINT_URL,
    }
    sqs = boto3.resource("sqs", **sqs_client_options)
    queue = sqs.get_queue_by_name(QueueName=settings.SERVICE_QUEUE)
    signal_handler = SignalHandler()
    wait_time_seconds = settings.AWS_SQS_CONSUMER_WAIT_TIME_SECONDS or 60
    while not signal_handler.received_signal:
        for message in queue.receive_messages(
            MaxNumberOfMessages=100, WaitTimeSeconds=wait_time_seconds
        ):
            body = json.loads(message.body)
            exchange_fn = exchanges[body.get("Subject")]
            exchange_fn(body.get("Message"))
            message.delete()



from django.core.exceptions import ValidationError
from rest_framework import exceptions as rest_exceptions

from .errors import get_error_message


class ApiErrorsMixin:
    """
    Mixin that transforms Django and Python exceptions into rest_framework ones.
    without the mixin, they return 500 status code which is not desired.
    """

    expected_exceptions = {
        ValueError: rest_exceptions.ValidationError,
        ValidationError: rest_exceptions.ValidationError,
        PermissionError: rest_exceptions.PermissionDenied,
    }

    def handle_exception(self, exc):
        if isinstance(exc, tuple(self.expected_exceptions.keys())):
            drf_exception_class = self.expected_exceptions[exc.__class__]
            drf_exception = drf_exception_class(get_error_message(exc))

            return super().handle_exception(drf_exception)

        return super().handle_exception(exc)
